@@ -51,6 +51,8 @@ impl From<Error> for IoError {
     }
 }
 
+impl std::error::Error for Error {}
+
 impl Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::result::Result<(), std::fmt::Error> {
         match self {
@@ -60,3 +62,13 @@ impl Display for Error {
         }
     }
 }
+
+// When io_error_more graduates from nightly these can use the NotSeekable kind directly
+#[cfg(unix)]
+pub(crate) fn seek_error() -> IoError {
+    IoError::from_raw_os_error(libc::ESPIPE)
+}
+#[cfg(not(unix))]
+pub(crate) fn seek_error() -> IoError {
+    IoError::new(ErrorKind::NotFound, "Cannot seek on stream")
+}
