@@ -1,9 +1,11 @@
-use crate::{is_fifo, Result};
+use crate::error::seek_error;
+use crate::{is_fifo, Error, Result};
+
 use std::convert::TryFrom;
 use std::ffi::{OsStr, OsString};
 use std::fmt::{self, Debug, Display};
 use std::fs::{File, OpenOptions};
-use std::io::{self, Result as IoResult, Stdout, Write};
+use std::io::{self, Result as IoResult, Seek, Stdout, Write};
 
 #[cfg(feature = "http")]
 use crate::http::{is_http, try_to_url, HttpWriter};
@@ -37,9 +39,12 @@ impl Output {
     }
 
     /// Contructs a new output either by opening/creating the file or for '-' returning stdout
-    /// The error is converted to a OsString so that stuctopt can show it to the user
+    ///
+    /// The error is converted to a [`OsString`](std::ffi::OsString) so that [stuctopt](https://docs.rs/structopt/latest/structopt/#custom-string-parsers) can show it to the user.
+    ///
+    /// It is recomended that you use [`TryFrom::try_from`] and [clap 3.0](https://docs.rs/clap/latest/clap/index.html) instead.
     pub fn try_from_os_str(path: &OsStr) -> std::result::Result<Self, std::ffi::OsString> {
-        TryFrom::try_from(path)
+        TryFrom::try_from(path).map_err(|e: Error| e.to_os_string(path))
     }
 
     /// Syncs the file to disk or closes any HTTP connections and returns any errors
@@ -100,10 +105,19 @@ impl Write for Output {
     }
 }
 
+impl Seek for Output {
+    fn seek(&mut self, pos: io::SeekFrom) -> IoResult<u64> {
+        match self {
+            Output::File(_, file) => file.seek(pos),
+            _ => Err(seek_error()),
+        }
+    }
+}
+
 impl TryFrom<&OsStr> for Output {
-    type Error = std::ffi::OsString;
-    fn try_from(file_name: &OsStr) -> std::result::Result<Self, std::ffi::OsString> {
-        Output::new(file_name).map_err(|e| e.to_os_string(file_name))
+    type Error = crate::Error;
+    fn try_from(file_name: &OsStr) -> Result<Self> {
+        Output::new(file_name)
     }
 }
 
@@ -127,10 +141,13 @@ impl SizedOutput {
         }
     }
 
-    /// Contructs a new output either by opening/creating the file or for '-' returning stdout
-    /// The error is converted to a OsString so that stuctopt can show it to the user
+    /// Contructs a new [`SizedOutput`] either by opening/creating the file or for '-' returning stdout
+    ///
+    /// The error is converted to a [`OsString`](std::ffi::OsString) so that [stuctopt](https://docs.rs/structopt/latest/structopt/#custom-string-parsers) can show it to the user.
+    ///
+    /// It is recomended that you use [`TryFrom::try_from`] and [clap 3.0](https://docs.rs/clap/latest/clap/index.html) instead.
     pub fn try_from_os_str(path: &OsStr) -> std::result::Result<Self, std::ffi::OsString> {
-        TryFrom::try_from(path)
+        TryFrom::try_from(path).map_err(|e: Error| e.to_os_string(path))
     }
 
     /// set the length of the file, either as the content-length header of the http put
@@ -163,9 +180,9 @@ impl SizedOutput {
 }
 
 impl TryFrom<&OsStr> for SizedOutput {
-    type Error = std::ffi::OsString;
-    fn try_from(file_name: &OsStr) -> std::result::Result<Self, std::ffi::OsString> {
-        SizedOutput::new(file_name).map_err(|e| e.to_os_string(file_name))
+    type Error = crate::Error;
+    fn try_from(file_name: &OsStr) -> Result<Self> {
+        SizedOutput::new(file_name)
     }
 }
 
